/*  Copyright (C) 2013 Zoltán Nagy
 *  Created by Zoltán Nagy on 3/13/13.
 *
 *  This program is free software: you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation, either version 3 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program. If not, see <http://www.gnu.org/licenses/>.
 */
#ifndef __mc_splitfunc_base_h__
#define __mc_splitfunc_base_h__ 1

/*   std includes   */
#include <numbers>


/*   project headers   */
#include <bits/duct-lorentzvector.h>
#include <bits/mc-hard-params.h>
#include <cxx/stackvector.h>



namespace duct {

  /*   Helper class to implement the default splitting functions for shower   */
  template<class _Type>
  struct splitting_functions;
  
  
  /*
   *    Final state splitting functions for shower evolution.
   */
  template<>
  struct splitting_functions<fin>
  {
    /*   small class to store the result for each channels   */
    struct result
    {
      int emitter, spectator;
      int fl, fm;
      double weight = 0.0;
    };
    
    /*   calculates all the possible splitting functions    */
    static auto splitfunc(const hard_params<fin>&, double, double) -> stackvector<result, 6u>;
    
    /*   calculates all the possible splitting functions    */
    static double splitfunc_sum(const hard_params<fin>&, double, double);
    
     
    /*   Azimuthal angle for the f -> f + g processes in the final state.   */
    static double azimuthal_angle(int, double, double, double, double, double);
  };
  
  
  /*
   *    Final state splitting functions for shower evolution.
   */
  template<>
  struct splitting_functions<ini>
  {
    /*   small class to store the result for each channels   */
    struct result
    {
      int emitter, spectator;
      int fl, fm;
      double weight = 0.0;
    };
    
    /*   calculates all the possible splitting functions    */
    static auto splitfunc(const hard_params<ini>&, double, double) -> stackvector<result, 6u>;
    
    /*   calculates all the possible splitting functions    */
    static std::pair<double, double> splitfunc_sum(const hard_params<ini>&, double, double);
    
     
    /*   Azimuthal angle for the f -> f + g processes in the final state.   */
    static double azimuthal_angle(double, double, double, double);
  };
  
  
 
}   /*   namespace duct    */

#endif

