/*  Copyright (C) 2015 Zoltán Nagy
 *  Created by Zoltán Nagy on 23/9/15.
 *
 *  This program is free software: you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation, either version 3 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program. If not, see <http://www.gnu.org/licenses/>.
 */
#ifndef __duct_cxx_span_h__
#define __duct_cxx_span_h__ 1

/*   std includes   */

/*   project headers   */

namespace duct {
  
  
  template<typename _Tp, std::size_t _Nrow, std::size_t ..._Rest>
  class span
  {
  public:
    /*    public types   */
    using size_type = std::size_t;
    using difference_type = std::ptrdiff_t;
    
    using value_type = _Tp;
    using pointer = _Tp *;
    using reference = _Tp&;
    using iterator = pointer;
    
    /*   constants    */
    static constexpr size_type rank = 1 + sizeof...(_Rest);
    static constexpr size_type size = (_Rest*...*_Nrow);
    
    /*   constructors   */
    constexpr explicit span() : _M_ptr{nullptr} {}
    constexpr explicit span(pointer p) : _M_ptr{p} {}
    
    template<std::size_t _Ext>
    constexpr explicit span(value_type (&arr)[_Ext]) : _M_ptr{arr} {
      static_assert(size <= _Ext, "span<_Tp, _Nrow, std::size_t ..._Rest> : The buffers is not sufficient enough.");
    }
    
    
    template<class _Viewable>
    constexpr explicit span(_Viewable&& v) : _M_ptr{v.data()} {}
    
    template<class U>
    constexpr explicit span(const span<U, _Nrow, _Rest...>& v) : _M_ptr{v.data()} {}
    
    /*   element access   */
    constexpr span<_Tp, _Rest...> operator[](difference_type k) const noexcept {
      return span<_Tp, _Rest...>{_M_ptr + (_Rest*...*k)};
    }
    
    /*   iterators   */
    constexpr iterator begin() const { return _M_ptr;}
    constexpr iterator end() const { return _M_ptr + size;}
    
    /*   observers   */
    constexpr pointer data() const { return _M_ptr;}
    
  private:
    pointer _M_ptr;
  };
  
  
  
  template<typename _Tp, std::size_t _Nrow>
  class span<_Tp, _Nrow>
  {
  public:
    /*    public types   */
    using size_type = std::size_t;
    using difference_type = std::ptrdiff_t;
    
    using value_type = _Tp;
    using pointer = _Tp *;
    using reference = _Tp&;
    using iterator = pointer;
    
    /*   constants    */
    static constexpr size_type rank = 1;
    static constexpr size_type size = _Nrow;
    
    /*   constructors   */
    constexpr explicit span() : _M_ptr{nullptr} {}
    constexpr explicit span(pointer p) : _M_ptr{p} {}
    
    template<std::size_t _Ext>
    constexpr explicit span(value_type (&arr)[_Ext]) : _M_ptr{arr} {
      static_assert(size <= _Ext, "span<_Tp, _Nrow> : The buffers is not sufficient enough.");
    }
    
    template<class _Viewable>
    constexpr explicit span(_Viewable&& v) : _M_ptr{v.data()} {}
    
    template<class U>
    constexpr explicit span(const span<U, _Nrow>& v) : _M_ptr{v.data()} {}
    
    /*   element access   */
    constexpr reference operator[](difference_type k) const noexcept {
      return _M_ptr[k];
    }
    
    /*   iterators   */
    constexpr iterator begin() const { return _M_ptr;}
    constexpr iterator end() const { return _M_ptr + size;}
    
    /*   observers   */
    constexpr pointer data() const { return _M_ptr;}
    
  private:
    pointer _M_ptr;
  };
  
}

#endif

