/*  Copyright (C) 2015 Zoltán Nagy
 *  Created by Zoltán Nagy on 3/24/15.
 *
 *  This program is free software: you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation, either version 3 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program. If not, see <http://www.gnu.org/licenses/>.
 */
#ifndef __cxx_misc_iterator_h__
#define __cxx_misc_iterator_h__ 1

/*   std includes   */
#include <utility>
#include <iterator>
#include <type_traits>


/*   project headers   */


namespace duct {
  
  
  
  /*   simple concept for inpot iterators (not complete!!!)    */
  template<typename I>
  concept InputIterator = std::input_iterator<I>;
  
}  //   namespace duct



/*   In this header I have some simple iterator related objects those are not available
 *   in the standard.  
 */
namespace duct {
  
  /*    iterators traversal tags  */
  struct inorder {};
  struct preorder {};
  struct postorder {};
  struct leaf {};
  
  
  
  /*    pair of iterator to be able to define ranges with various traversal   */
  template<typename _Iter>
  class iterator_range : std::pair<_Iter, _Iter>
  {
    using pair_t = std::pair<_Iter, _Iter>;
   
  public:
    /*  public types  */
    using iterator = _Iter;
    
    /*    constructors  */
    iterator_range() = default;
    iterator_range(const pair_t& src) : pair_t{src} {}
    iterator_range(pair_t&& src) : pair_t{std::move(src)} {}
    
    iterator_range(const iterator& i1, const iterator& i2)
    : pair_t{i1, i2} {}

    iterator_range(iterator&& i1, iterator&& i2)
    : pair_t{std::move(i1), std::move(i2)} {}

    /*   begin() / end() operators  */
    iterator begin() const { return this->first;}
    iterator end() const { return this->second;}
  };
  
  template<typename _Iter>
  auto make_range(_Iter&& b, _Iter&& e) {
    return iterator_range<std::remove_reference_t<_Iter>>{std::forward<_Iter>(b), std::forward<_Iter>(e)};
  }
  
}

#endif
