/*  rs-resum-user.h
 *  Created by Zoltan Nagy on 30.01.20.
 *  Copyright © 2020 Zoltán Nagy. All rights reserved.
 *
 *  This program is free software: you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation, either version 3 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program. If not, see <http://www.gnu.org/licenses/>.
 */
#ifndef __rs_resum_user_h__
#define __rs_resum_user_h__ 1




/*   std includes    */

/*   project headers    */
#include <bits/mc-basic-user.h>
#include <bits/rs-resum-exp.h>




namespace duct {
  namespace resum {
    
    
    /*   Abstract class to define the most general user interface.
     *   Every analyser should be inhereted from this class.
     *   This class adds the userfunc(...) function and defines the
     *   full resum_user<Tag> abstract class.
     */
    template<ProcessTag _Tag>
    class resum_user : public virtual user_base
    {
    public:
      /*   Virtual destructor   */
      virtual ~resum_user() = default;
      
      
      /*   This analyser routine is used in most of the time.
       *   Usually we need access only to the hard, threshold and showered stages.
       */
      virtual void userfunc(const resummation_exponent<_Tag>&) = 0;
    };
    
    
    /*   In general this is the user class that one should directly use
     *   in a calculation. This comes weith many goodies to handle histograms.
     */
    template<ProcessTag _Tag, class ..._DistBook>
    class basic_resum_user : public basic_user_core<_DistBook...>, public resum_user<_Tag>
    {};
    

  
  }   //   namespace resum
}   //   namespace duct


#endif
